import os
import json
import pathlib
import time
from packaging import version
from kabaret import flow
from kabaret.flow_entities.store import EntityStore

from libreflow import _version as libreflow_version
from libreflow.rouge import _version as project_version

from libreflow.baseflow import (
    ProjectSettings as BaseProjectSettings,
    MultisiteConfig,
    SynchronizeFiles,
    KitsuConfig,
    LoginPage,
    LogOut
)
from libreflow.baseflow.film import FilmCollection
from libreflow.baseflow.task_manager import TaskManager
from libreflow.baseflow.users import Users, UserProfile, UserEnvironment
from libreflow.baseflow.site import GotoCurrentSiteQueue
from libreflow.baseflow.runners import (
    Blender,
    SessionWorker,
    DefaultEditor,
    DefaultRunners
)

from .entity_manager import EntityManager
from .film import Film
from .shot import Sequence, Shot
from .task import Task
from .users import User
from .file import (
    TrackedFile,
    TrackedFolder,
    Revision,
    TrackedFolderRevision,
)


class ProjectSettings(BaseProjectSettings):

    task_manager      = flow.Child(TaskManager)

    libreflow_version = flow.Param('0.0.0').ui(editable=False)
    project_version   = flow.Param('0.0.0').ui(editable=False)


class Synchronization(flow.Object):

    ICON = ('icons.libreflow', 'sync')
    
    _project = flow.Parent()
    synchronize_files = flow.Child(SynchronizeFiles)
    jobs = flow.Child(GotoCurrentSiteQueue)

    def summary(self):
        nb_waiting_jobs = self.root().project().get_current_site().count_jobs(
            status='WAITING'
        )
        
        if nb_waiting_jobs > 0:
            return (
                "<font color=#D5000D><b>"
                f"{nb_waiting_jobs} job(s) waiting"
                "</b></font>"
            )


class Admin(flow.Object):

    ICON = ('icons.gui', 'user-admin')

    _project = flow.Parent()

    project_settings = flow.Child(ProjectSettings)
    users = flow.Child(Users).ui(show_filter=True)
    user_environment = flow.Child(UserEnvironment).ui(expanded=False)
    multisites = flow.Child(MultisiteConfig).ui(label='Site management')
    kitsu = flow.Child(KitsuConfig).ui(label='Kitsu settings')
    default_applications = flow.Child(DefaultRunners).ui(expanded=False)
    login_page = flow.Child(LoginPage).ui(hidden=True)

    entity_manager = flow.Child(EntityManager)


class Project(flow.Object):
    '''
    This class defines the project root.
    '''
    
    log_out_action  = flow.Child(LogOut).ui(label='Log out')
    user            = flow.Child(UserProfile)
    synchronization = flow.Child(Synchronization)
    films           = flow.Child(FilmCollection).ui(expanded=True)
    admin           = flow.Child(Admin)

    _RUNNERS_FACTORY = None

    @classmethod
    def _injection_provider(cls, slot_name, default_type):
        if slot_name == 'libreflow.baseflow.film.Film':
            return Film
        elif slot_name == 'libreflow.baseflow.shot.Sequence':
            return Sequence
        elif slot_name == 'libreflow.baseflow.shot.Shot':
            return Shot
        elif slot_name == 'libreflow.baseflow.file.ManagedTask':
            return Task
        elif slot_name == 'libreflow.baseflow.file.TrackedFile':
            return TrackedFile
        elif slot_name == 'libreflow.baseflow.file.TrackedFolder':
            return TrackedFolder
        elif slot_name == 'libreflow.baseflow.file.Revision':
            return Revision
        elif slot_name == 'libreflow.baseflow.file.TrackedFolderRevision':
            return TrackedFolderRevision
        elif slot_name == 'libreflow.baseflow.users.User':
            return User
    
    def get_contextual_view(self, context_name):
        if context_name == 'settings':
            return self.admin.project_settings.settings
    
    def get_default_contextual_edits(self, context_name):
        if context_name == 'settings':
            return dict(path_format='{film}/{shot}/{file}_{revision}')

    def get_root(self, alternative=None):
        '''
        alternative can be used if root_dir.get() returns None
        '''
        root_dir = self.admin.multisites.root_dir.get()
        
        if root_dir is None and alternative is not None:
            root_dir = alternative
        
        return root_dir
    
    def get_temp_folder(self):
        return self.admin.multisites.temp_dir.get()
    
    def project_settings_folder(self):
        return pathlib.Path(pathlib.Path.home(), '.libreflow', self.name())

    def set_user_name(self, username):
        project_settings_folder = self.project_settings_folder()

        if not os.path.exists(project_settings_folder):
            os.makedirs(project_settings_folder)

        user_file = os.path.join(project_settings_folder, 'current_user.json')

        with open(user_file, 'w+') as f:
            user_config = dict(username=username)
            json.dump(user_config, f)
        
        self.user.current_user_id.touch()
    
    def get_user_name(self):
        '''
        Returns the name of the current user.
        '''
        return self.user.current_user_id.get()

    def get_user(self, name=None):
        '''
        Returns the user of this project with the given name.
        '''
        if name is None:
            name = self.get_user_name()
        
        if not self.admin.users.has_mapped_name(name):
            return None
        
        return self.admin.users[name]
    
    def get_default_file_presets(self):
        return self.admin.project_settings.default_files

    def user_settings_folder(self):
        return pathlib.Path(self.project_settings_folder(), self.get_user_name())
    
    def get_current_site(self):
        '''
        Returns the current working site.
        '''
        return self.get_working_sites()[
            self.admin.multisites.current_site_name.get()
        ]
    
    def get_exchange_site(self):
        '''
        Returns the current exchange site.
        '''
        return self.get_exchange_sites()[
            self.admin.multisites.exchange_site_name.get()
        ]

    def get_working_sites(self):
        return self.admin.multisites.working_sites

    def get_exchange_sites(self):
        return self.admin.multisites.exchange_sites

    def get_entity_manager(self):
        return self.admin.entity_manager

    def get_entity_store(self):
        return self.admin.entity_manager.store
    
    def get_task_manager(self):
        return self.admin.project_settings.task_manager
    
    def get_sync_manager(self):
        return self.synchronization.synchronize_files
    
    def get_action_value_store(self):
        return self.get_entity_manager().action_value_store
    
    def kitsu_config(self):
        return self.admin.kitsu
    
    def kitsu_api(self):
        return self.admin.kitsu.gazu_api
    
    def kitsu_bindings(self):
        return self.admin.kitsu.bindings
    
    def get_factory(self):
        self.ensure_runners_loaded()
        return self._RUNNERS_FACTORY
    
    def ensure_runners_loaded(self):
        session = self.root().session()
        subprocess_manager = session.get_actor('SubprocessManager')

        if self._RUNNERS_FACTORY is None:
            self._RUNNERS_FACTORY = subprocess_manager.create_new_factory(
                'Libreflow.rouge Tools'
            )
            self._register_runners()

        subprocess_manager.ensure_factory(self._RUNNERS_FACTORY)
    
    def update_kitsu_host(self, server_url):
        kitsu_api = self.kitsu_api()
        kitsu_api.set_server_url(server_url)
        kitsu_api.set_host(server_url + '/api')

        return kitsu_api.host_is_valid()
    
    def show_login_page(self):
        kitsu_api = self.kitsu_api()
        valid_host = kitsu_api.host_is_valid()

        if not valid_host:
            logged_in = False
        else:
            logged_in = kitsu_api.current_user_logged_in()

        if not logged_in:
            try:
                f = open('%s/kitsu_config.json' % self.user_settings_folder(), 'r')
            except IOError:
                logged_in = False
            else:
                kitsu_config = json.load(f)
                kitsu_api.set_host(kitsu_config['kitsu_host'])
                kitsu_api.set_tokens(kitsu_config['tokens'])

                logged_in = kitsu_api.current_user_logged_in()

        return not kitsu_api.host_is_valid() or not logged_in
    
    def log_in(self, kitsu_login, kitsu_password):
        kitsu_api = self.kitsu_api()

        self.update_kitsu_host(self.admin.kitsu.server_url.get())
        kitsu_user_data = kitsu_api.log_in(kitsu_login, kitsu_password)

        if kitsu_user_data is None:
            return 1
        
        kitsu_id = kitsu_user_data['user']['id']

        # Set current user
        # user_name = self.admin.users.get_user_name(kitsu_login)
        user = self.admin.users.get_user_by_uid(kitsu_id)

        if user is None:
            # Create user profile if not registered
            user = self.admin.users.add_user(kitsu_id, kitsu_login)

            if user is None:
                return 2

            # user_name = user.name()
        
        self.set_user_name(user.name())

        # Create user folder at first login
        user_settings_folder = self.user_settings_folder()
        if not os.path.exists(user_settings_folder):
            os.makedirs(user_settings_folder)

        # Save authentification tokens
        tokens = kitsu_api.get_tokens()
        kitsu_config = {}
        kitsu_config['tokens'] = tokens
        kitsu_config['kitsu_host'] = kitsu_api.get_host()

        if user_settings_folder is not None:
            with open('%s/kitsu_config.json' % user_settings_folder, 'w+') as f:
                json.dump(kitsu_config, f)

        return 0
    
    def log_out(self):
        self.kitsu_api().log_out()
    
    def update_user_environment(self):
        self.admin.user_environment.update()
    
    def update_user_last_visit(self):
        user_login = self.get_user_name()
        requiredVersion = self.get_required_versions()

        if not user_login or not requiredVersion:
            return

        users = self.admin.users

        if user_login not in users.mapped_names():
            return
        user = users[user_login]

        user._last_visit.set(time.time())
        for v in requiredVersion:
            if v[0] == 'libreflow.rouge':
                user._last_project_used_version.set(v[1])
            elif v[0] == 'libreflow':
                user._last_libreflow_used_version.set(v[1])

    def get_required_versions(self):
        '''
        Returns a list of dependencies
        [dependecyName, currentVersion, requiredVersion, updateNeeded(0:no|1:yes minor|2: yes major)],[]
        '''
        versions = []

        libreflow_cur_version = version.parse(
            libreflow_version.get_versions()['version']
        )
        libreflow_req_version = version.parse(
            self.admin.project_settings.libreflow_version.get()
        )
        
        if libreflow_cur_version < libreflow_req_version \
            and ((libreflow_cur_version.major < libreflow_req_version.major) or \
                (libreflow_cur_version.minor < libreflow_req_version.minor)):
            # VERY IMPORTANT UPDATE
            libreflow_needs_update = 2
        elif libreflow_cur_version < libreflow_req_version:
            # MINOR UPDATE
            libreflow_needs_update = 1
        else:
            # NO UDPATE
            libreflow_needs_update = 0

        versions.append([
            'libreflow',
            str(libreflow_cur_version),
            str(libreflow_req_version),
            libreflow_needs_update]
        )
        
        project_cur_version = version.parse(
            project_version.get_versions()['version']
        )
        project_req_version = version.parse(
            self.admin.project_settings.project_version.get()
        )

        if project_cur_version < project_req_version \
            and ((project_cur_version.major < project_req_version.major) or \
                (project_cur_version.minor < project_req_version.minor)):
            # VERY IMPORTANT UPDATE
            project_needs_update = 2
        elif project_cur_version < project_req_version:
            # MINOR UPDATE
            project_needs_update = 1
        else:
            # NO UDPATE
            project_needs_update = 0

       
        versions.append([
            'libreflow.rouge',
            str(project_cur_version),
            str(project_req_version),
            project_needs_update
        ])
    
        for v in versions:
            print(v)
        return versions
    
    def get_project_thumbnail(self):
        image = self.admin.project_settings.project_thumbnail.get()
        return image
    
    def touch(self):
        super(Project, self).touch()
        self.ensure_runners_loaded()
        self.update_user_environment()
    
    def _register_runners(self):
        self._RUNNERS_FACTORY.ensure_runner_type(Blender)
        self._RUNNERS_FACTORY.ensure_runner_type(SessionWorker)
        self._RUNNERS_FACTORY.ensure_runner_type(DefaultEditor)
    
    def _fill_ui(self, ui):
        if self._RUNNERS_FACTORY is None:
            self.ensure_runners_loaded()
        
        self.touch()

        if self.show_login_page():
            ui['custom_page'] = 'libreflow.baseflow.LoginPageWidget'



from kabaret import flow
from kabaret.app.actors.flow.generic_home_flow import ProjectsMap
from kabaret.app.actors.flow.generic_home_flow import AbstractHomeRoot


class ClassicHome(flow.Object):
    """
    Access the classic home with right click on the oid on top of the page
    Useful to add projects or change status
    """

    projects = flow.Child(ProjectsMap).ui(
        auto_fit=False, columns_width=(50, 20), expanded=True
    )


class Home(flow.SessionObject):
    """
    This Home is completely overrided by the custom_page
    designed in the custom_home.ui
    """

    ClassicHome = flow.Child(ClassicHome)

    def _fill_ui(self, ui):
        ui['custom_page'] = 'libreflow.rouge.custom_home.ui.ProjectHomePageWidget'

    def get_projects(self):
        return self.root().flow_actor.get_projects_info()


class CustomHomeRoot(AbstractHomeRoot):

    Home = flow.Child(Home)


import time
import timeago
import datetime
from packaging import version
from kabaret import flow
from libreflow.baseflow.users import User as BaseUser


class User(BaseUser):

    last_visit                   = flow.Computed()
    libreflow_version            = flow.Computed().ui(label='libreflow')
    project_version              = flow.Computed().ui(label='libreflow.rouge')

    _last_visit                  = flow.IntParam(0)
    _last_libreflow_used_version = flow.Param(None)
    _last_project_used_version   = flow.Param(None)

    def compute_child_value(self, child_value):
        if child_value is self.last_visit:
            if self._last_visit.get() == 0:
                self.last_visit.set('never')
            else:
                last_connection = datetime.datetime.fromtimestamp(
                    self._last_visit.get()
                )
                now = datetime.datetime.now()
                self.last_visit.set(timeago.format(last_connection, now))
        elif child_value is self.libreflow_version:
            requiered_version = version.parse(
                self.root().project().admin.project_settings.libreflow_version.get()
            )
            user_current_version = self._last_libreflow_used_version.get()

            if not user_current_version:
                self.libreflow_version.set('Unknown')
            else:
                user_current_version = version.parse(user_current_version)
                if requiered_version > user_current_version:
                    self.libreflow_version.set(
                        '%s (!)' % str(user_current_version)
                    )
                else:
                    self.libreflow_version.set(
                        '%s' % str(user_current_version)
                    )
        elif child_value is self.project_version:
            requiered_version = version.parse(
                self.root().project().admin.project_settings.project_version.get()
            )
            user_current_version = self._last_project_used_version.get()

            if not user_current_version:
                self.project_version.set('Unknown')
            else:
                user_current_version = version.parse(user_current_version)
                if requiered_version > user_current_version:
                    self.project_version.set(
                        '%s (!)' % str(user_current_version)
                    )
                else:
                    self.project_version.set(
                        '%s' % str(user_current_version)
                    )



import six
import os
from kabaret.app.ui.gui.widgets.flow.flow_view import (
    CustomPageWidget,
    QtWidgets,
    QtCore,
    QtGui,
)


class ProjectThumbnailWidget(QtWidgets.QAbstractButton):

    def __init__(self, width, height, image_path=None, alt_text='', parent=None, image=None):
        super(ProjectThumbnailWidget, self).__init__(parent)
        self.setThumbnail = True
        self.alt_text = alt_text

        if image:
            ba = QtCore.QByteArray.fromBase64(bytes(image.split(',')[1], 'utf-8'))
            pixmap = QtGui.QPixmap()
            pixmap.loadFromData(ba, 'PNG')
            self.thumbnail = pixmap
        elif image_path:
            print('loading thumbnail from path')
            self.thumbnail = QtGui.QPixmap(image_path)
        else:
            self.setThumbnail = False

        self.width = width
        self.height = height

    def sizeHint(self):
        return QtCore.QSize(self.width, self.height)

    def paintEvent(self, event):
        QPainter = QtGui.QPainter()
        QPainter.begin(self)

        if self.setThumbnail:
            QPainter.drawPixmap(0, 0, self.width, self.height, self.thumbnail)
        else:
            rect = QtCore.QRect(0, 0, self.width - 1, self.height - 1)

            QPainter.drawRect(rect)
            font = QPainter.font()
            font.setPixelSize(int(self.height / 3))
            QPainter.setFont(font)

            QPainter.drawText(
                rect, QtCore.Qt.AlignCenter | QtCore.Qt.AlignVCenter, self.alt_text
            )
        QPainter.end()


class ProjectWidget(QtWidgets.QWidget):

    def __init__(self, parent=None, project=None, HomePageWidget=None):
        super(ProjectWidget, self).__init__(parent)
        self.grid_layout = QtWidgets.QGridLayout()
        self.HomePageWidget = HomePageWidget
        
        try:
            project_thumbnail = self.HomePageWidget.session.cmds.Flow.call(
                '/%s' % project, 'get_project_thumbnail', {}, {}
            )
        except AttributeError:
            project_thumbnail = None
        
        # Project
        self.thumbnail_button = ProjectThumbnailWidget(
            600, 150, image_path=None, alt_text=project, image=project_thumbnail
        )

        self.thumbnail_button.clicked.connect(
            lambda project=project, button='thumbnail': self.on_project_button_clicked(
                project, button
            )
        )

        # Bookmarks
        self.fav_button = ProjectThumbnailWidget(
            128, 32,  alt_text='Bookmarks'
        )
        self.fav_button.clicked.connect(
            lambda project=project, button='fav': self.on_project_button_clicked(
                project, button
            )
        )

        self.grid_layout.addWidget(self.thumbnail_button, 0, 0, 3, 1)
        self.grid_layout.addWidget(self.fav_button, 0, 1)

        try:
            requiered_versions = self.HomePageWidget.session.cmds.Flow.call(
                '/%s' % project, 'get_required_versions', {}, {}
            )
            
            start_line = 3

            for r in requiered_versions:
                color = '#74787a'
                if r[3] > 0:
                    color = '#b9c2c8'
                required_update = ''
                if r[3] == 1:
                    required_update = ' <strong>Update to %s available</strong>' % r[2]
                elif r[3] == 2:
                    required_update = ' <font color=\'red\'><strong>Update required to %s</strong></font>' % r[2]
                text = '<font color=\'%s\'>%s %s %s</font>' % (color, r[0], r[1], required_update)
                t = QtWidgets.QLabel(text)
                t.setMargin(20)
                t.setAlignment(QtCore.Qt.AlignCenter | QtCore.Qt.AlignVCenter)
                self.grid_layout.addWidget(t, start_line, 0, 3, 1)

                start_line += 1
        except AttributeError:
            pass

        # Update users last connexion
        try:
            self.HomePageWidget.session.cmds.Flow.call(
                '/%s' % project, 'update_user_last_visit', {}, {}
            )
        except AttributeError:
            pass
        
        self.setLayout(self.grid_layout)

    def on_project_button_clicked(self, project, button):
        if button == 'thumbnail':
            oid = '/%s' % project
        elif button == 'fav':
            oid = '/%s/user' % project
        
        self.HomePageWidget.session.cmds.Flow.call('/' + project, 'touch', {}, {})
        self.HomePageWidget.page.goto(oid)


class ProjectHomePageWidget(CustomPageWidget):
    
    def build(self):
        self.projectsButtons = {}
        project_list = QtWidgets.QVBoxLayout()

        for name, infos in self.session.cmds.Flow.call('/Home', 'get_projects', {}, {}):
            if infos['status'] == 'Archived':
                continue

            project_list.addWidget(ProjectWidget(None, name, self))
        
        self.setLayout(project_list)



import argparse
from qtpy import QtCore

from kabaret.app.ui.gui import KabaretStandaloneGUISession
from kabaret.script_view import ScriptView
from kabaret.app.actors.flow import Flow

from libreflow.utils.kabaret.jobs.jobs_view import JobsView
from libreflow.utils.kabaret.jobs.jobs_actor import Jobs
from libreflow.utils.kabaret.subprocess_manager import SubprocessManager, SubprocessView

from .custom_home import CustomHomeRoot


class SessionGUI(KabaretStandaloneGUISession):

    def __init__(self, session_name='Standalone', tick_every_ms=10, debug=False, script_view=True, jobs_view=True):
        super(SessionGUI, self).__init__(session_name, tick_every_ms, debug)
        self._script_view = script_view
        self._jobs_view = jobs_view

    def register_view_types(self):
        super(SessionGUI, self).register_view_types()

        type_name = self.register_view_type(SubprocessView)
        self.add_view(
            type_name,
            view_id='Processes',
            hidden=not self.debug_mode,
            area=QtCore.Qt.RightDockWidgetArea,
        )

        if self._script_view:
            type_name = self.register_view_type(ScriptView)
            self.add_view(
                type_name,
                hidden=not self.debug_mode,
                area=QtCore.Qt.RightDockWidgetArea
            )
        if self._jobs_view:
            type_name = self.register_view_type(JobsView)
            self.add_view(
                type_name,
                hidden=not self.debug_mode,
                area=QtCore.Qt.RightDockWidgetArea,
            )

    def _create_actors(self):
        super(SessionGUI, self)._create_actors()
        
        Flow(self, CustomHomeRootType=CustomHomeRoot)
        subprocess_manager = SubprocessManager(self)
        jobs = Jobs(self)

    @staticmethod
    def parse_command_line_args(args):
        (
            session_name,
            host,
            port,
            cluster_name,
            db,
            password,
            debug,
            remaining_args,
        ) = KabaretStandaloneGUISession.parse_command_line_args(args)

        parser = argparse.ArgumentParser(
            description='Libreflow.rouge Session Arguments'
        )

        parser.add_argument(
            '-u', '--user', dest='user'
        )
        parser.add_argument(
            '-s', '--site', dest='site'
        )
        parser.add_argument(
            '-j', '--jobs_filter', dest='jobs_filter'
        )

        values, remaining_args = parser.parse_known_args(remaining_args)

        return (
            session_name,
            host, port, cluster_name,
            db, password, debug,
            values.user, values.site, values.jobs_filter, remaining_args
        )



from kabaret import flow
from libreflow.baseflow.shot import (
    ShotCollection,
    Shot as BaseShot,
    Sequence as BaseSequence
)
from libreflow.baseflow.task import ManagedTaskCollection


class Shot(BaseShot):
    
    tasks = flow.Child(ManagedTaskCollection).ui(
        expanded=True
    )

    def ensure_tasks(self):
        """
        Creates the tasks of this shot based on the task
        templates of the project, skipping any existing task.
        """
        mgr = self.root().project().get_task_manager()

        for dt in mgr.default_tasks.mapped_items():
            if not self.tasks.has_mapped_name(dt.name()) and not dt.optional.get():
                t = self.tasks.add(dt.name())
                t.enabled.set(dt.enabled.get())
        
        self.tasks.touch()


class Shots(ShotCollection):

    def add(self, name, object_type=None):
        """
        Adds a shot to the global shot collection, and creates
        its tasks.
        """
        s = super(Shots, self).add(name, object_type)
        s.ensure_tasks()


class Sequence(BaseSequence):
    
    shots = flow.Child(Shots).ui(expanded=True)



import pathlib

from libreflow.baseflow.file import (
    TrackedFile            as BaseTrackedFile,
    TrackedFolder          as BaseTrackedFolder,
    Revision               as BaseRevision,
    TrackedFolderRevision  as BaseTrackedFolderRevision,
    FileSystemMap          as BaseFileSystemMap
)
from libreflow.utils.flow import get_contextual_dict


class Revision(BaseRevision):
    pass


class TrackedFolderRevision(BaseTrackedFolderRevision):
    pass


class TrackedFile(BaseTrackedFile):
    pass


class TrackedFolder(BaseTrackedFolder):
    pass


class FileSystemMap(BaseFileSystemMap):
    
    def add_file(self, name, extension, display_name=None, base_name=None, tracked=False, default_path_format=None):
        if default_path_format is None:
            default_path_format = get_contextual_dict(self, 'settings').get(
                'path_format', None
            )
        return super(FileSystemMap, self).add_file(name, extension, display_name, base_name, tracked, default_path_format)

    def add_folder(self, name, display_name=None, base_name=None, tracked=False, default_path_format=None):
        if default_path_format is None:
            default_path_format = get_contextual_dict(self, 'settings').get(
                'path_format', None
            )
        return super(FileSystemMap, self).add_folder(name, display_name, base_name, tracked, default_path_format)



import setuptools
import versioneer
import os

readme = os.path.normpath(os.path.join(__file__, "..", "README.md"))
with open(readme, "r", encoding="utf-8") as fh:
    long_description = fh.read()

long_description += "\n\n"

changelog = os.path.normpath(os.path.join(__file__, "..", "CHANGELOG.md"))
with open(changelog, "r", encoding="utf-8") as fh:
    long_description = fh.read()


setuptools.setup(
    cmdclass=versioneer.get_cmdclass(),
    name="libreflow.rouge",
    version=versioneer.get_version(),
    author="Flavio Perez",
    author_email="flavio@lfs.coop",
    description="A rouge flow",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/lfs.coop/libreflow/libreflow.rouge",
    license="LGPLv3+",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)",
        "Operating System :: OS Independent",
    ],
    keywords="kabaret cgwire kitsu gazu animation pipeline libreflow",
    install_requires=["libreflow>=2.1.1"],
    python_requires=">=3.7",
    packages=setuptools.find_packages("src"),
    package_dir={"": "src"},
    package_data={
        "": ["*.css", "*.png", "*.svg", "*.gif", "*.abc", "*.aep", "*.ai", "*.blend", "*.jpg", "*.kra", "*.mov", "*.psd", "*.txt", "*.usd", "*.fbx", "*.json", "*.obj"],
    },

)


from kabaret import flow
from libreflow.utils.kabaret.flow_entities.entities import GlobalEntityCollection
from libreflow.utils.flow import ActionValueStore
from libreflow.baseflow.entity_manager import EntityManager as BaseEntityManager


class EntityManager(BaseEntityManager):

    films              = flow.Child(GlobalEntityCollection)
    sequences          = flow.Child(GlobalEntityCollection)
    shots              = flow.Child(GlobalEntityCollection)
    tasks              = flow.Child(GlobalEntityCollection)
    files              = flow.Child(GlobalEntityCollection)
    revisions          = flow.Child(GlobalEntityCollection)
    sync_statutes      = flow.Child(GlobalEntityCollection)
    
    action_value_store = flow.Child(ActionValueStore)

    def get_film_collection(self):
        return self.films
    
    def get_sequence_collection(self):
        return self.sequences

    def get_shot_collection(self):
        return self.shots
    
    def get_task_collection(self):
        return self.tasks

    def get_file_collection(self):
        return self.files
    
    def get_revision_collection(self):
        return self.revisions
    
    def get_sync_status_collection(self):
        return self.sync_statutes



import sys
import os

os.environ['QT_MAC_WANTS_LAYER'] = '1'

from kabaret.app.ui.gui.styles.gray import GrayStyle
from libreflow.resources.icons import libreflow, status
from libreflow.resources import file_templates

from .session import SessionGUI

GrayStyle()


def main(argv):
    (
        session_name,
        host,
        port,
        cluster_name,
        db,
        password,
        debug,
        user,
        site,
        jobs_filter,
        remaining_args,
    ) = SessionGUI.parse_command_line_args(argv)

    if site:
        os.environ['KABARET_SITE_NAME'] = site
    if user:
        os.environ['USER_NAME'] = user
    if jobs_filter:
        os.environ['JOBS_DEFAULT_FILTER'] = jobs_filter
    else:
        os.environ['JOBS_DEFAULT_FILTER'] = site

    session = SessionGUI(session_name=session_name, debug=debug)
    session.cmds.Cluster.connect(host, port, cluster_name, db, password)

    session.start()
    session.close()


if __name__ == '__main__':
    main(sys.argv[1:])